function [ak, bk, df, y] = icompLS_akbk(s, fk, win, fs, iter)
%
% Compute iteratively the parameters of first order complex polynomial
% model using Least Squares.
% 
%  ----INPUT PARAMETERS----
%    s: signal
%    fk: estimated frequencies, where the analysis is performed
%    win: the window
%    fs: sampling frequency
%    iter: Number of iterations
%
%  ----OUTPUT PARAMENTERS----
%    ak: amplitude of harmonics
%    bk: slope of harmonics
%    df: frequency mismatch
%    y: reconstructed signal
%
% (c) Yannis Pantazis, 2008, csd, uoc
%     mail: pantazis@csd.uoc.gr


N = (length(s)-1)/2; % length of s must be odd
K = length(fk);
n = -N:N;

% INITIALIZATION
E = exp(j*n'*2*pi*fk/fs); % dimension of E: (2N+1)x(K)
E = [E repmat(n',1,K).*E]; % dimension of E:(2N+1)x(2K)
Ew = repmat(win,1,2*K).*E;

R = Ew'*Ew;

if cond(R)<10^10
    x = R\(Ew'*(win .* s)); % solution of LS
else
    fprintf(1, 'CAUTION: Bad condition of matrix.\n');
    ak = zeros(K,1);
    bk = zeros(K,1);
    df = zeros(1,K);
    y = E*[ak;bk];
    return;
end

ak = x(1:K); % amplitudes
bk = x(K+1:2*K); % slopes

% ITERATION
df = zeros(1,K);
for i = 1:iter
    df = df + fs/(2*pi)*((imag(bk).*real(ak) - imag(ak).*real(bk))./abs(ak).^2)';

    E = exp(j*n'*2*pi*(fk+df)/fs); % dimension of E: (2N+1)x(K)
    E = [E repmat(n',1,K).*E]; % dimension of E:(2N+1)x(2K)
    Ew = repmat(win,1,2*K).*E;

    R = Ew'*Ew;
    
    if cond(R)<10^10
        x = R\(Ew'*(win .* s)); % solution of LS
    else
        fprintf(1, 'CAUTION: Bad condition of matrix.\n');
        ak = zeros(K,1);
        bk = zeros(K,1);
        df = zeros(1,K);
        y = E*[ak;bk];
        return;
    end

    ak = x(1:K); % amplitudes
    bk = x(K+1:2*K); % slopes
end

y = E*[ak;bk]; % reconstructed signal

%figure(5);plot(real(s));hold on;plot(real(y),'r');hold off;pause;